#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
This package implements the (q,s)-test.
"""

from .qstest import qstest
from .quality_functions import *
from .size_functions import * 

__author__ = """Sadamori Kojaku (sadamori.koujaku@bristol.ac.uk)"""
